#include <p18f2680.h>
#include <usart.h>
#include "ser_buff.h"

/***** 18F2680 Configuration *************************************************/
#pragma config OSC = IRCIO67	// Use internal oscillator
#pragma config WDT = OFF	// Turn off watchdog timer
#pragma config LVP = OFF	// Disable low voltage programming
#pragma config DEBUG = OFF	// Disable debug mode

/***** Macros and defines ****************************************************/
// shortcuts for each of the I/O pins 
#define red     PORTBbits.RB0
#define orange  PORTBbits.RB1
#define yellow  PORTBbits.RB2
#define blue    PORTBbits.RB3
#define green   PORTBbits.RB4


// and macros to turn them on and off
#define _on(a)	a = 1
#define _off(a)	a = 0

/***** Function Prototypes ***************************************************/
void highp_handler (void);
void lowp_handler (void);
void wait(void);


/***** Setup for serial communications ***************************************/

//#define BUF_SIZE 25

#pragma udata bigdata

Circ_buff tx_buff, rx_buff;
unsigned char state_b, state_b_last;

char pitchtable[5];

char note;

#pragma code highp_interrupt = 0x08
void highp_int (void) {
    _asm goto highp_handler _endasm
}

#pragma code lowp_interrupt = 0x18
void lowp_int (void) {
    _asm goto lowp_handler _endasm
}


#pragma code
/***** Program Code **********************************************************/

// High priority interrupt handler
#pragma interrupt highp_handler
void highp_handler (void) {
    static char dat;

    if( PIR1bits.RCIF == 1) 
        // just read the data and write it to the buffer... facile!
        write_char( &rx_buff, ReadUSART() );

        // should add a bit here to check if buffer is full to do flow control
    
    else if( PIR1bits.TXIF == 1 ) {
        if( read_char( &tx_buff, &dat ) == BMESG_SUCCESS ) 
            TXREG = dat;
        else 
            // we cant simply shut off the TX here because the last character
            // is possibly still transferring.
            if( TXSTAbits.TRMT == 1 )
                TXSTAbits.TXEN = 0;
    }

}
// Low priority interrupt handler
#pragma interruptlow lowp_handler
void lowp_handler (void) {
    unsigned char c;
    static unsigned char buff[10];
  
   // current input pin has changed
    if( INTCONbits.RBIF == 1 ) {
        INTCONbits.RBIF = 0;
    }
}


void wait(void)
{
    int a;
    for(a=0;a<254;a++) {}
    for(a=0;a<254;a++) {}
    for(a=0;a<254;a++) {}
    for(a=0;a<254;a++) {}
    for(a=0;a<254;a++) {}
    for(a=0;a<254;a++) {}
}

void handle_portb(void) {
    unsigned char c;
    unsigned char i;
    unsigned char buff[10];
//    _on(red);

        state_b_last = state_b;
        state_b = PORTB;
        c = state_b ^ state_b_last;
        for( i = 0; i < 5; i++ ) {
            if( (c >> i) & 1 ) {
                if ( (state_b >> i) & 1 ) {
                    buff[0] = 0x80;
                }
                else {
                    buff[0] = 0x90;
                }
                    buff[1] = pitchtable[i];
                    buff[2] = 100;
                    while ( write_string( &tx_buff, buff, 3 )
                            != BMESG_SUCCESS )  {}

                TXSTAbits.TXEN = 1;
            }
        }
//    _off(red);
}

void handle_serial(void) {
    unsigned char c, i;
    unsigned int total;
    char buff[10];

    if( read_char( &rx_buff, &c ) != BMESG_SUCCESS )
        return;

    switch( c ) {
        case '1':
//            if( green == 0 )
//                _on(green);
//            else
//                _off(green);
            break;
        default:
            break;
    }

    return;
}

void main (void)
{
    char i;
    OSCCON  = 0x72;  // Internal 8MHz osc 
	
// set up serial port and high-priority interrupts for the serial port
    baudUSART(BAUD_16_BIT_RATE &
              BAUD_WAKEUP_OFF &
              BAUD_AUTO_OFF);
			
    OpenUSART (USART_TX_INT_ON &
               USART_RX_INT_ON &
               USART_ASYNCH_MODE &
               USART_EIGHT_BIT &
               USART_CONT_RX &
               USART_BRGH_HIGH, 63);

//    TXSTA = 0b00100010;
//    RCSTA = 0b10000000;
//    BAUDCON = 0b00000000;
//    SPBRGH = 0x0;
//    SPBRG = 63;     // magic number for 31250bps signal, considering 8 MHz clock

//    RCSTAbits.SPEN = 1;
    

    IPR1bits.RCIP = 1;	// Make receive interrupt high priority
    IPR1bits.TXIP = 1;	// Make transmit interrupt high priority

// clear circular buffer
    init_buffer( &tx_buff );
//    init_buffer( &rx_buff );


// Set up timer0
    T0CONbits.T08BIT = 1;	  // Use 16-bit timer mode (default)
    T0CONbits.T0CS = 0;     // Use the Fosc/4 as a clock source
    T0CONbits.PSA = 0;      // Use the prescaler
    T0CONbits.T0PS2 = 1;    // Set the prescaler value
    T0CONbits.T0PS1 = 1;    // 110 = /128 prescale 
    T0CONbits.T0PS0 = 0;

// Set up analog inputs
    PIE1bits.ADIE = 0;      // turn off adc interrupt
    PIR1bits.ADIF = 0;
    ADCON1 = 0b00001100;    // AN0-AN3 are anaog inputs
                            // Voltage references use Vss, Vdd
    ADCON0 = 0b00000001;    // Select AN0, but don't do anything yet
    ADCON2 = 0b00011101;    // Left Justified, 6 TAD, Fosc/16

// Set up port A
//    ADCON1 = 0b00001111;  // turn off ADC functionality
    TRISA  = 0b00000111;  // 0=output, 1=input
//    PORTA  = 0b00000000;  // value of the output pins

// Set up port B
    TRISB = 0b00111111;  // 0=output, 1=input
    PORTB = 0b00000000;  // value of the output pins
    INTCON2bits.NOT_RBPU = 0;  // Turn on pull-up resistors for inputs

    state_b = 0;
    
    pitchtable[0] = 72;
    pitchtable[1] = 74;
    pitchtable[2] = 76;
    pitchtable[3] = 77;
    pitchtable[4] = 79;


//    INTCON2bits.RBIP = 0;  // Make port B interrupt priority low
//    INTCONbits.RBIE = 1;   // Enable port B interrupts

// Turn on interrupts 
    RCONbits.IPEN = 1;    // Enable interrupt priority
    INTCONbits.GIEH = 1;  // Enable all high priority interrupts
    INTCONbits.GIEL = 1;  // Enable all low priority interrupts

// Main loop
    while (1)
    {
//        if( get_avail_space( &rx_buff ) != BUFFER_LENGTH )
//            handle_serial();
        handle_portb();
    }
}
