//#include <stdio.h>      // only for testing
#include <string.h>     // for memcpy()
#include "ser_buff.h"

#ifdef _18CXX
#pragma code
#endif

unsigned char init_buffer( Circ_buff *buff ) {
    unsigned char i;

    // NOT THREAD SAFE, CALL FROM MAIN FUNCTION AND DISABLE INTERRUPTS HERE!!!
    buff->read_pos = 0;
    buff->write_pos = 0;
    // CAN RE-ENABLE INTERRUPTS HERE
    //
    return BMESG_SUCCESS;
}

unsigned char get_avail_space( Circ_buff *buff ) {
    if( buff->read_pos == buff->write_pos )
        return BUFFER_LENGTH - 1;
    if( ( buff->read_pos == 0 && buff->write_pos == BUFFER_LENGTH - 1 ) ||
        ( buff->read_pos - buff->write_pos == 1 ) )
        return 0;
    
    if( buff->read_pos < buff->write_pos )
        return BUFFER_LENGTH - buff->write_pos + buff->read_pos - 1;
    else //( read_pos > write_pos )
        return buff->read_pos - buff->write_pos - 1;
}

unsigned char read_char( Circ_buff *buff, char *data ) {
    // THREAD SAFE IF ONLY CALLED BY A SINGLE THREAD
    // For instance, this should only be called by the main function, or by
    // a single interrupt, but never both.
    if( buff->read_pos == buff->write_pos )
        return BMESG_NO_DATA;
	    
    *data = buff->buffer[ buff->read_pos ];
    buff->read_pos = (buff->read_pos + 1) & (BUFFER_LENGTH -1);
    return BMESG_SUCCESS;
}

unsigned char write_char( Circ_buff *buff, char data ) {
    // THREAD SAFE IF ONLY CALLED BY A SINGLE THREAD
    // For instance, this should only be called by the main function, or by
    // a single interrupt, but never both.
    if( get_avail_space( buff ) == 0 )
        return BMESG_NOT_ENOUGH_SPACE;
    
    buff->buffer[buff->write_pos] = data;
    buff->write_pos = (buff->write_pos + 1) & (BUFFER_LENGTH - 1);
    return BMESG_SUCCESS;
}

unsigned char read_string( Circ_buff *buff, char *data, unsigned char *length) {
    // THREAD SAFE IF ONLY CALLED BY A SINGLE THREAD
    // For instance, this should only be called by the main function, or by
    // a single interrupt, but never both.

    // length is the maximum length to read bach (size of *data) on call
    // and on successful read is overwritten with the acutal length of data read

    if( buff->read_pos == buff->write_pos )
        return BMESG_NO_DATA;
   
    // check to see if the data wraps; if it doesnt, copy in one fell sweep,
    // otherwise copy in two calls
 
    return BMESG_NOT_IMPLEMENTED;
}

unsigned char write_string( Circ_buff *buff, char *data, unsigned char length ) {
    // THREAD SAFE IF ONLY CALLED BY A SINGLE THREAD
    // For instance, this should only be called by the main function, or by
    // a single interrupt, but never both.
    size_t length_a, length_b;
    char a, b;

    if( length > (BUFFER_LENGTH - 1) )
        return BMESG_WOULD_NEVER_FIT;

    if( length > get_avail_space( buff ) )
        return BMESG_NOT_ENOUGH_SPACE;

    length_a = length;  // This is because memcpy requires a size_t (which is
                        // declared as unsiged short long int by the C18
                        // compiler), but we use a char for the length to
                        // make things faster (the buffer can be at max 256
                        // characters long).

    if( buff->write_pos < buff->read_pos ) {
        // In this case, the free space does not wrap across the boundary and
        // the data can be copied intact.
        memcpy( &buff->buffer[buff->write_pos], data, length_a );
        buff->write_pos = (buff->write_pos + length) & (BUFFER_LENGTH - 1);
    }
    else {
        if( length < ( BUFFER_LENGTH - buff->write_pos ) ) {
            // If the data does not wrap across the boundary, copy it in one
            // motion.
            // note: we don't need to check for the case where we are writing
            // data to the end of the buffer and read_pos = 0, because that
            // would mean the data being copied is too big for the buffer.
            memcpy( &buff->buffer[buff->write_pos], data, length_a );
            buff->write_pos = (buff->write_pos + length) & (BUFFER_LENGTH - 1);
        }
        else {
            // fill to the end of the buffer, then copy the remainder to the
            // beginning of the buffer.
            length_a =  (BUFFER_LENGTH - buff->write_pos);
            length_b = (length - BUFFER_LENGTH + buff->write_pos);

            memcpy( &buff->buffer[buff->write_pos], data, length_a );
            memcpy( &buff->buffer[0], &data[length_a], length_b ); 
            buff->write_pos = length_b;
        }
    }

    return BMESG_SUCCESS;
}


